import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { Visibility_$type } from "igniteui-webcomponents-core";
import { IgcToolCommandEventArgs } from "./igc-tool-command-event-args";
import { IgcToolActionPerformedEventArgs } from "./igc-tool-action-performed-event-args";
import { ToolAction } from "./ToolAction";
import { TypeRegistrar } from "igniteui-webcomponents-core";
import { NamePatcher, getAllPropertyNames, fromSpinal, toSpinal, brushToString, stringToBrush, ensureBool, toBrushCollection, fromBrushCollection, brushCollectionToString, ensureEnum, enumToString, initializePropertiesFromCss } from "igniteui-webcomponents-core";
import { ToolActionCollection as ToolActionCollection_internal } from "./ToolActionCollection";
import { SyncableObservableCollection$1 } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
/**
 * Represents base class for all tool actions in toolbar
*/
export let IgcToolActionComponent = /*@__PURE__*/ (() => {
    class IgcToolActionComponent extends IgcHTMLElement {
        createImplementation() {
            return null;
        }
        /**
         * @hidden
         */
        get i() {
            return this._implementation;
        } /**
                                     * @hidden
                                     */
        static _createFromInternal(internal) {
            if (!internal) {
                return null;
            }
            if (!internal.$type) {
                return null;
            }
            let name = internal.$type.name;
            let externalName = "Igc" + name + "Component";
            if (!TypeRegistrar.isRegistered(externalName)) {
                return null;
            }
            return TypeRegistrar.create(externalName);
        }
        onImplementationCreated() {
        }
        constructor() {
            super();
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this._actions = null;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            this._performed = null;
            this._performed_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this.onImplementationCreated();
            if (this._initializeAdapters) {
                this._initializeAdapters();
            }
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        connectedCallback() {
            if (super["connectedCallback"]) {
                super["connectedCallback"]();
            }
            if (this.i.connectedCallback) {
                this.i.connectedCallback();
            }
            if (!this._attached) {
                this._attached = true;
                this._flushQueuedAttributes();
            }
        }
        disconnectedCallback() {
            if (super["disconnectedCallback"]) {
                super["disconnectedCallback"]();
            }
            if (this.i.disconnectedCallback) {
                this.i.disconnectedCallback();
            }
            if (this._attached) {
                this._attached = false;
            }
        }
        static get observedAttributes() {
            if (IgcToolActionComponent._observedAttributesIgcToolActionComponent == null) {
                let names = getAllPropertyNames(IgcToolActionComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolActionComponent._observedAttributesIgcToolActionComponent = names;
            }
            return IgcToolActionComponent._observedAttributesIgcToolActionComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        get id() {
            return this.i.ee;
        }
        /**
         * Gets the sub actions associated with this tool action.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let innerColl = this.i.actions;
                if (!innerColl) {
                    innerColl = new ToolActionCollection_internal();
                }
                this._actions = coll._fromInner(innerColl);
            }
            return this._actions;
        }
        set actions(v) {
            if (this._actions !== null) {
                this._actions._setSyncTarget(null);
                this._actions = null;
            }
            let coll = new IgcToolActionCollection();
            this._actions = coll._fromOuter(v);
            let syncColl = new SyncableObservableCollection$1(ToolAction.$type);
            let innerColl = this.i.actions;
            if (!innerColl) {
                innerColl = new ToolActionCollection_internal();
            }
            syncColl._inner = innerColl;
            syncColl.clear();
            this._actions._setSyncTarget(syncColl);
        }
        /**
         * Gets or sets the width to use for this tool action.
        */
        get width() {
            return this.i.ck;
        }
        set width(v) {
            this.i.ck = +v;
            this._a("width", this.i.ck);
        }
        get parentId() {
            return this.i.eo;
        }
        set parentId(v) {
            this.i.eo = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted before.
         * Subsequent actions will continue to insert before the target action, following
         * the original action that set this property. If using this property do not use the
         * AfterId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get beforeId() {
            return this.i.dk;
        }
        set beforeId(v) {
            this.i.dk = v;
        }
        /**
         * Gets or sets the Id of the action that this action will be inserted after.
         * Subsequent actions will continue to insert after the target action, following
         * the original action that set this property. If using this property do not use the
         * BeforeId property.
         * This property is only relevant during initial insertion into the toolbar. Changing the property
         * after the action has already been added will have no effect.
        */
        get afterId() {
            return this.i.dh;
        }
        set afterId(v) {
            this.i.dh = v;
        }
        /**
         * Sets to the Id of the action that this action will overlay itself on to.
         * The purpose of this property is to allow users to override actions automatically generated by
         * toolbar targets such as the DataChart. These generated actions all have unique Ids associated
         * with them that can be used for the OverlayId.
         * These overlaid actions will
         * not
         * replace the generated one but will have its properties
         * overlaid onto the generated actions.
        */
        get overlayId() {
            return this.i.ej;
        }
        set overlayId(v) {
            this.i.ej = v;
        }
        get commandId() {
            return this.i.dq;
        }
        set commandId(v) {
            this.i.dq = v;
        }
        get commandArgument() {
            return this.i.dm;
        }
        set commandArgument(v) {
            this.i.dm = v;
        }
        get commandArgumentValue() {
            return this.i.cm;
        }
        set commandArgumentValue(v) {
            this.i.cm = v;
        }
        get paddingLeft() {
            return this.i.cg;
        }
        set paddingLeft(v) {
            this.i.cg = +v;
            this._a("paddingLeft", this.i.cg);
        }
        get actualPaddingLeft() {
            return this.i.bz;
        }
        set actualPaddingLeft(v) {
            this.i.bz = +v;
            this._a("actualPaddingLeft", this.i.bz);
        }
        get paddingRight() {
            return this.i.ch;
        }
        set paddingRight(v) {
            this.i.ch = +v;
            this._a("paddingRight", this.i.ch);
        }
        get actualPaddingRight() {
            return this.i.b0;
        }
        set actualPaddingRight(v) {
            this.i.b0 = +v;
            this._a("actualPaddingRight", this.i.b0);
        }
        get paddingBottom() {
            return this.i.cf;
        }
        set paddingBottom(v) {
            this.i.cf = +v;
            this._a("paddingBottom", this.i.cf);
        }
        get actualPaddingBottom() {
            return this.i.by;
        }
        set actualPaddingBottom(v) {
            this.i.by = +v;
            this._a("actualPaddingBottom", this.i.by);
        }
        get paddingTop() {
            return this.i.ci;
        }
        set paddingTop(v) {
            this.i.ci = +v;
            this._a("paddingTop", this.i.ci);
        }
        get actualPaddingTop() {
            return this.i.b1;
        }
        set actualPaddingTop(v) {
            this.i.b1 = +v;
            this._a("actualPaddingTop", this.i.b1);
        }
        /**
         * Gets the actual hover background used by the tool action.
        */
        get actualHoverBackground() {
            return brushToString(this.i.gv);
        }
        set actualHoverBackground(v) {
            this.i.gv = stringToBrush(v);
            this._a("actualHoverBackground", brushToString(this.i.gv));
        }
        /**
         * Gets or sets the hover background for this tool action.
        */
        get hoverBackground() {
            return brushToString(this.i.g3);
        }
        set hoverBackground(v) {
            this.i.g3 = stringToBrush(v);
            this._a("hoverBackground", brushToString(this.i.g3));
        }
        /**
         * Gets the actual background used by the tool action.
        */
        get actualBackground() {
            return brushToString(this.i.gs);
        }
        set actualBackground(v) {
            this.i.gs = stringToBrush(v);
            this._a("actualBackground", brushToString(this.i.gs));
        }
        /**
         * Gets or sets the background color of the tool action.
        */
        get background() {
            return brushToString(this.i.g0);
        }
        set background(v) {
            this.i.g0 = stringToBrush(v);
            this._a("background", brushToString(this.i.g0));
        }
        /**
         * Gets the actual label text color.
        */
        get actualTextColor() {
            return brushToString(this.i.gz);
        }
        set actualTextColor(v) {
            this.i.gz = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.gz));
        }
        /**
         * Gets or sets the label text color.
        */
        get textColor() {
            return brushToString(this.i.g7);
        }
        set textColor(v) {
            this.i.g7 = stringToBrush(v);
            this._a("textColor", brushToString(this.i.g7));
        }
        /**
         * Gets the actual label text color.
        */
        get actualSubtitleTextColor() {
            return brushToString(this.i.gy);
        }
        set actualSubtitleTextColor(v) {
            this.i.gy = stringToBrush(v);
            this._a("actualSubtitleTextColor", brushToString(this.i.gy));
        }
        /**
         * Gets or sets the label text color.
        */
        get subtitleTextColor() {
            return brushToString(this.i.g6);
        }
        set subtitleTextColor(v) {
            this.i.g6 = stringToBrush(v);
            this._a("subtitleTextColor", brushToString(this.i.g6));
        }
        /**
         * Gets the actual label text color when the tool is disabled.
        */
        get actualDisabledTextColor() {
            return brushToString(this.i.gt);
        }
        set actualDisabledTextColor(v) {
            this.i.gt = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.gt));
        }
        /**
         * Gets or sets the label text color when the tool is disabled.
        */
        get disabledTextColor() {
            return brushToString(this.i.g1);
        }
        set disabledTextColor(v) {
            this.i.g1 = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.g1));
        }
        /**
         * Gets the actual font used.
        */
        get actualTextStyle() {
            return this.i.ab;
        }
        set actualTextStyle(v) {
            this.i.ab = v;
        }
        /**
         * Gets or sets the font to use.
        */
        get textStyle() {
            if (this.i.ae == null) {
                return null;
            }
            return this.i.ae.fontString;
        }
        set textStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.ae = fi;
            this._a("textStyle", this.i.ae != null ? this.i.ae.fontString : "");
        }
        /**
         * Gets the actual font used.
        */
        get actualSubtitleTextStyle() {
            return this.i.aa;
        }
        set actualSubtitleTextStyle(v) {
            this.i.aa = v;
        }
        /**
         * Gets or sets the font to use.
        */
        get subtitleTextStyle() {
            if (this.i.ad == null) {
                return null;
            }
            return this.i.ad.fontString;
        }
        set subtitleTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.ad = fi;
            this._a("subtitleTextStyle", this.i.ad != null ? this.i.ad.fontString : "");
        }
        get isDisabled() {
            return this.i.az;
        }
        set isDisabled(v) {
            this.i.az = ensureBool(v);
            this._a("isDisabled", this.i.az);
        }
        get isHighlighted() {
            return this.i.a0;
        }
        set isHighlighted(v) {
            this.i.a0 = ensureBool(v);
            this._a("isHighlighted", this.i.a0);
        }
        get name() {
            return this.i.eh;
        }
        set name(v) {
            this.i.eh = v;
        }
        /**
         * Gets or sets the title of the tool.
        */
        get title() {
            return this.i.ez;
        }
        set title(v) {
            this.i.ez = v;
        }
        /**
         * Gets or sets the subtitle of the tool.
        */
        get subtitle() {
            return this.i.et;
        }
        set subtitle(v) {
            this.i.et = v;
        }
        get iconName() {
            return this.i.d4;
        }
        set iconName(v) {
            this.i.d4 = v;
        }
        get iconCollectionName() {
            return this.i.dz;
        }
        set iconCollectionName(v) {
            this.i.dz = v;
        }
        /**
         * Gets or sets the icon viewbox left.
        */
        get iconViewBoxLeft() {
            return this.i.cb;
        }
        set iconViewBoxLeft(v) {
            this.i.cb = +v;
            this._a("iconViewBoxLeft", this.i.cb);
        }
        /**
         * Gets or sets the icon viewbox top.
        */
        get iconViewBoxTop() {
            return this.i.cc;
        }
        set iconViewBoxTop(v) {
            this.i.cc = +v;
            this._a("iconViewBoxTop", this.i.cc);
        }
        /**
         * Gets or sets the icon viewbox width.
        */
        get iconViewBoxWidth() {
            return this.i.cd;
        }
        set iconViewBoxWidth(v) {
            this.i.cd = +v;
            this._a("iconViewBoxWidth", this.i.cd);
        }
        /**
         * Gets or sets the icon viewbox height.
        */
        get iconViewBoxHeight() {
            return this.i.ca;
        }
        set iconViewBoxHeight(v) {
            this.i.ca = +v;
            this._a("iconViewBoxHeight", this.i.ca);
        }
        /**
         * Gets or sets the icon fill color.
        */
        get iconFill() {
            return brushToString(this.i.g4);
        }
        set iconFill(v) {
            this.i.g4 = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.g4));
        }
        /**
         * Gets the actual icon fill color.
        */
        get actualIconFill() {
            return brushToString(this.i.gw);
        }
        set actualIconFill(v) {
            this.i.gw = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.gw));
        }
        /**
         * Gets or sets the icon fill colors.
        */
        get iconFillColors() {
            return fromBrushCollection(this.i.c);
        }
        set iconFillColors(v) {
            this.i.c = toBrushCollection(v);
            this._a("iconFillColors", brushCollectionToString(this.i.c));
        }
        /**
         * Gets or sets the icon stroke color.
        */
        get iconStroke() {
            return brushToString(this.i.g5);
        }
        set iconStroke(v) {
            this.i.g5 = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.g5));
        }
        /**
         * Gets the actual icon stroke color.
        */
        get actualIconStroke() {
            return brushToString(this.i.gx);
        }
        set actualIconStroke(v) {
            this.i.gx = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.gx));
        }
        /**
         * Gets or sets the icon stroke colors.
        */
        get iconStrokeColors() {
            return fromBrushCollection(this.i.d);
        }
        set iconStrokeColors(v) {
            this.i.d = toBrushCollection(v);
            this._a("iconStrokeColors", brushCollectionToString(this.i.d));
        }
        /**
         * Gets or sets the icon viewbox width.
        */
        get iconStrokeWidth() {
            return this.i.b9;
        }
        set iconStrokeWidth(v) {
            this.i.b9 = +v;
            this._a("iconStrokeWidth", this.i.b9);
        }
        /**
         * Gets or sets the width of the icon.
        */
        get iconWidth() {
            return this.i.ce;
        }
        set iconWidth(v) {
            this.i.ce = +v;
            this._a("iconWidth", this.i.ce);
        }
        /**
         * Gets the actual icon width used.
        */
        get actualIconWidth() {
            return this.i.bx;
        }
        set actualIconWidth(v) {
            this.i.bx = +v;
            this._a("actualIconWidth", this.i.bx);
        }
        /**
         * Gets or sets the height of the icon.
        */
        get iconHeight() {
            return this.i.b8;
        }
        set iconHeight(v) {
            this.i.b8 = +v;
            this._a("iconHeight", this.i.b8);
        }
        /**
         * Gets the actual icon height used.
        */
        get actualIconHeight() {
            return this.i.bw;
        }
        set actualIconHeight(v) {
            this.i.bw = +v;
            this._a("actualIconHeight", this.i.bw);
        }
        /**
         * Gets or sets the tool action visibility.
        */
        get visibility() {
            return this.i.g9;
        }
        set visibility(v) {
            this.i.g9 = ensureEnum(Visibility_$type, v);
            this._a("visibility", enumToString(Visibility_$type, this.i.g9));
        }
        /**
         * Gets or sets if the tool action will close its parent dropdown when executed by the user.
        */
        get closeOnExecute() {
            return this.i.as;
        }
        set closeOnExecute(v) {
            this.i.as = ensureBool(v);
            this._a("closeOnExecute", this.i.as);
        }
        get highlightWidth() {
            return this.i.b7;
        }
        set highlightWidth(v) {
            this.i.b7 = +v;
            this._a("highlightWidth", this.i.b7);
        }
        get actualHighlightWidth() {
            return this.i.bv;
        }
        set actualHighlightWidth(v) {
            this.i.bv = +v;
            this._a("actualHighlightWidth", this.i.bv);
        }
        get highlightRadius() {
            return this.i.b6;
        }
        set highlightRadius(v) {
            this.i.b6 = +v;
            this._a("highlightRadius", this.i.b6);
        }
        get actualHighlightRadius() {
            return this.i.bu;
        }
        set actualHighlightRadius(v) {
            this.i.bu = +v;
            this._a("actualHighlightRadius", this.i.bu);
        }
        get highlightColor() {
            return brushToString(this.i.g2);
        }
        set highlightColor(v) {
            this.i.g2 = stringToBrush(v);
            this._a("highlightColor", brushToString(this.i.g2));
        }
        get actualHighlightColor() {
            return brushToString(this.i.gu);
        }
        set actualHighlightColor(v) {
            this.i.gu = stringToBrush(v);
            this._a("actualHighlightColor", brushToString(this.i.gu));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolActionComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        cleanUp() {
            this.i.e3();
        }
        openSubMenu() {
            this.i.fy();
        }
        closeSubmenu() {
            this.i.e5();
        }
        exportVisualData() {
            let iv = this.i.cn();
            return (iv);
        }
        exportSerializedVisualData() {
            let iv = this.i.du();
            return (iv);
        }
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
        get performed() {
            return this._performed;
        }
        set performed(ev) {
            if (this._performed_wrapped !== null) {
                this.i.performed = delegateRemove(this.i.performed, this._performed_wrapped);
                this._performed_wrapped = null;
                this._performed = null;
            }
            this._performed = ev;
            this._performed_wrapped = (o, e) => {
                let outerArgs = new IgcToolActionPerformedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforePerformed) {
                    this.beforePerformed(this, outerArgs);
                }
                if (this._performed) {
                    this._performed(this, outerArgs);
                }
            };
            this.i.performed = delegateCombine(this.i.performed, this._performed_wrapped);
            ;
        }
    }
    IgcToolActionComponent._observedAttributesIgcToolActionComponent = null;
    return IgcToolActionComponent;
})();
