import { fromSize, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "igniteui-webcomponents-core";
import { FontInfo } from "igniteui-webcomponents-core";
import { WebComponentRenderer, PortalManager } from "igniteui-webcomponents-core";
import { delegateCombine, delegateRemove } from "igniteui-webcomponents-core";
import { CollectionAdapter } from "igniteui-webcomponents-core";
import { NotifyCollectionChangedAction } from "igniteui-webcomponents-core";
import { RegisterElementHelper } from "igniteui-webcomponents-core";
import { ToolPanel } from "./ToolPanel";
import { BaseControlTheme_$type } from "igniteui-webcomponents-core";
import { IgcHTMLElement } from "igniteui-webcomponents-core";
import { ToolPanelOrientation_$type } from './ToolPanelOrientation';
import { IgcToolActionComponent } from "./igc-tool-action-component";
import { IgcToolActionCollection } from "./igc-tool-action-collection";
import { IgcToolPanelContentRefreshedEventArgs } from "./igc-tool-panel-content-refreshed-event-args";
import { IgcToolCommandEventArgs } from './igc-tool-command-event-args';
import { NestedActionMode_$type } from './NestedActionMode';
export let IgcToolPanelComponent = /*@__PURE__*/ (() => {
    class IgcToolPanelComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
            this.i.notifySizeChanged();
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
            this.i.notifySizeChanged();
        }
        get width() {
            return this._width;
        }
        /**
         * A collection of manually added actions for the editor.
        */
        get actions() {
            if (this._actions === null) {
                let coll = new IgcToolActionCollection();
                let inner = coll._innerColl;
                inner.addListener((sender, e) => {
                    switch (e.action) {
                        case NotifyCollectionChangedAction.Add:
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Remove:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            break;
                        case NotifyCollectionChangedAction.Replace:
                            this._actionsAdapter.removeManualItemAt(e.oldStartingIndex);
                            this._actionsAdapter.insertManualItem(e.newStartingIndex, e.newItems.item(0));
                            break;
                        case NotifyCollectionChangedAction.Reset:
                            this._actionsAdapter.clearManualItems();
                            break;
                    }
                });
                this._actions = coll;
            }
            return this._actions;
        }
        constructor() {
            super();
            this._implementation = null;
            this.contentActions = [];
            /**
            * The actions actually present in the editor. Do not directly modify this array.
            * This array's contents can be modified by using the actions property or providing child content.
            */
            this.actualActions = [];
            this._actions = null;
            this._actionsAdapter = null;
            this._disconnected = false;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._contentRefreshed = null;
            this._contentRefreshed_wrapped = null;
            this._onCommand = null;
            this._onCommand_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._implementation.externalObject = this;
            this._portalManager = new PortalManager("toolPanelContent");
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {}, this._portalManager);
            this._webComponentWrapper = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._webComponentWrapper);
            this._webComponentWrapper.setStyleProperty("display", "block");
            this._webComponentWrapper.setStyleProperty("width", "100%");
            this._webComponentWrapper.setStyleProperty("height", "100%");
            this._actionsAdapter = new CollectionAdapter(this.contentActions, this.i.actions, this.actualActions, (c) => c.i, (i) => {
                if (this._webComponentRenderer && this._webComponentRenderer.rootWrapper.getNativeElement().parentElement) {
                    i._styling(this, this, this);
                }
            }, (i) => { });
            this.i.provideContainer(this._webComponentRenderer);
            this.i.notifySizeChanged();
            let mut = new MutationObserver((list) => {
                for (var mutation of list) {
                    if (mutation.type == 'childList') {
                        this.updateContentProperties();
                    }
                }
            });
            mut.observe(this, {
                childList: true
            });
            this._webComponentRenderer.addSizeWatcher(() => {
                this.i.notifySizeChanged();
            });
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        updateContentProperties() {
            this.contentActions.length = 0;
            for (var i = 0; i < this.children.length; i++) {
                if (this.children[i] instanceof IgcToolActionComponent) {
                    this.contentActions.push(this.children[i]);
                }
            }
            if (this._actionsAdapter != null) {
                this._actionsAdapter.notifyContentChanged();
            }
        }
        createImplementation() {
            return new ToolPanel();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-tool-panel");
            this.classList.add("igc-tool-panel");
            var rootWrapper = this._webComponentRenderer.rootWrapper;
            var rootElement = rootWrapper.getNativeElement();
            this.appendChild(rootElement);
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            for (let i = 0; i < this.actualActions.length; i++) {
                let s = this.actualActions[i];
                s._styling(this, this, this);
            }
            this.afterContentInit();
        }
        afterContentInit() {
            this.i.notifySizeChanged();
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcToolPanelComponent._observedAttributesIgcToolPanelComponent == null) {
                let names = getAllPropertyNames(IgcToolPanelComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcToolPanelComponent._observedAttributesIgcToolPanelComponent = names;
            }
            return IgcToolPanelComponent._observedAttributesIgcToolPanelComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcToolPanelComponent._isElementRegistered) {
                IgcToolPanelComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcToolPanelComponent.htmlTagName, IgcToolPanelComponent);
            }
        }
        /**
     * Gets or sets if the tool actions should display their submenus on mouse hover.
    */
        get showOnHover() {
            return this.i.al;
        }
        set showOnHover(v) {
            this.i.al = ensureBool(v);
            this._a("showOnHover", this.i.al);
        }
        /**
         * Gets or sets the text style for the panel.
        */
        get cellTextStyle() {
            if (this.i.ab == null) {
                return null;
            }
            return this.i.ab.fontString;
        }
        set cellTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.ab = fi;
            this._a("cellTextStyle", this.i.ab != null ? this.i.ab.fontString : "");
        }
        /**
         * Gets or sets the subtitle text style for the panel.
        */
        get subtitleTextStyle() {
            if (this.i.ad == null) {
                return null;
            }
            return this.i.ad.fontString;
        }
        set subtitleTextStyle(v) {
            let fi = new FontInfo();
            fi.fontString = v;
            this.i.ad = fi;
            this._a("subtitleTextStyle", this.i.ad != null ? this.i.ad.fontString : "");
        }
        /**
         * Gets or sets the base built in theme to use for the checkbox.
        */
        get baseTheme() {
            return this.i.u;
        }
        set baseTheme(v) {
            this.i.u = ensureEnum(BaseControlTheme_$type, v);
            this._a("baseTheme", enumToString(BaseControlTheme_$type, this.i.u));
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get backgroundColor() {
            return brushToString(this.i.ha);
        }
        set backgroundColor(v) {
            this.i.ha = stringToBrush(v);
            this._a("backgroundColor", brushToString(this.i.ha));
        }
        get actualBackgroundColor() {
            return brushToString(this.i.gp);
        }
        set actualBackgroundColor(v) {
            this.i.gp = stringToBrush(v);
            this._a("actualBackgroundColor", brushToString(this.i.gp));
        }
        /**
         * Gets or sets the background color for submenus.
        */
        get submenuBackgroundColor() {
            return brushToString(this.i.ho);
        }
        set submenuBackgroundColor(v) {
            this.i.ho = stringToBrush(v);
            this._a("submenuBackgroundColor", brushToString(this.i.ho));
        }
        get actualSubmenuBackgroundColor() {
            return brushToString(this.i.g3);
        }
        set actualSubmenuBackgroundColor(v) {
            this.i.g3 = stringToBrush(v);
            this._a("actualSubmenuBackgroundColor", brushToString(this.i.g3));
        }
        /**
         * Gets or sets the group header background color.
        */
        get groupHeaderBackgroundColor() {
            return brushToString(this.i.he);
        }
        set groupHeaderBackgroundColor(v) {
            this.i.he = stringToBrush(v);
            this._a("groupHeaderBackgroundColor", brushToString(this.i.he));
        }
        get actualGroupHeaderBackgroundColor() {
            return brushToString(this.i.gt);
        }
        set actualGroupHeaderBackgroundColor(v) {
            this.i.gt = stringToBrush(v);
            this._a("actualGroupHeaderBackgroundColor", brushToString(this.i.gt));
        }
        /**
         * Gets or sets the separator background color.
        */
        get separatorBackgroundColor() {
            return brushToString(this.i.hn);
        }
        set separatorBackgroundColor(v) {
            this.i.hn = stringToBrush(v);
            this._a("separatorBackgroundColor", brushToString(this.i.hn));
        }
        get actualSeparatorBackgroundColor() {
            return brushToString(this.i.g2);
        }
        set actualSeparatorBackgroundColor(v) {
            this.i.g2 = stringToBrush(v);
            this._a("actualSeparatorBackgroundColor", brushToString(this.i.g2));
        }
        /**
         * Gets or sets the group header separator background color.
        */
        get groupHeaderSeparatorBackgroundColor() {
            return brushToString(this.i.hf);
        }
        set groupHeaderSeparatorBackgroundColor(v) {
            this.i.hf = stringToBrush(v);
            this._a("groupHeaderSeparatorBackgroundColor", brushToString(this.i.hf));
        }
        get actualGroupHeaderSeparatorBackgroundColor() {
            return brushToString(this.i.gu);
        }
        set actualGroupHeaderSeparatorBackgroundColor(v) {
            this.i.gu = stringToBrush(v);
            this._a("actualGroupHeaderSeparatorBackgroundColor", brushToString(this.i.gu));
        }
        /**
         * Gets or sets the hover background color.
        */
        get hoverBackgroundColor() {
            return brushToString(this.i.hj);
        }
        set hoverBackgroundColor(v) {
            this.i.hj = stringToBrush(v);
            this._a("hoverBackgroundColor", brushToString(this.i.hj));
        }
        get actualHoverBackgroundColor() {
            return brushToString(this.i.gy);
        }
        set actualHoverBackgroundColor(v) {
            this.i.gy = stringToBrush(v);
            this._a("actualHoverBackgroundColor", brushToString(this.i.gy));
        }
        /**
         * Gets or sets the color to use for the text of the component.
        */
        get textColor() {
            return brushToString(this.i.hq);
        }
        set textColor(v) {
            this.i.hq = stringToBrush(v);
            this._a("textColor", brushToString(this.i.hq));
        }
        get actualTextColor() {
            return brushToString(this.i.g5);
        }
        set actualTextColor(v) {
            this.i.g5 = stringToBrush(v);
            this._a("actualTextColor", brushToString(this.i.g5));
        }
        get actualSubtitleTextColor() {
            return brushToString(this.i.g4);
        }
        set actualSubtitleTextColor(v) {
            this.i.g4 = stringToBrush(v);
            this._a("actualSubtitleTextColor", brushToString(this.i.g4));
        }
        /**
         * Gets or sets the color to use for the subtitle of the component.
        */
        get subtitleTextColor() {
            return brushToString(this.i.hp);
        }
        set subtitleTextColor(v) {
            this.i.hp = stringToBrush(v);
            this._a("subtitleTextColor", brushToString(this.i.hp));
        }
        /**
         * Gets or sets the color to use for the text in disabled tool actions.
        */
        get disabledTextColor() {
            return brushToString(this.i.hd);
        }
        set disabledTextColor(v) {
            this.i.hd = stringToBrush(v);
            this._a("disabledTextColor", brushToString(this.i.hd));
        }
        get actualDisabledTextColor() {
            return brushToString(this.i.gs);
        }
        set actualDisabledTextColor(v) {
            this.i.gs = stringToBrush(v);
            this._a("actualDisabledTextColor", brushToString(this.i.gs));
        }
        /**
         * Gets or sets the text color for group header tool actions.
        */
        get groupHeaderTextColor() {
            return brushToString(this.i.hh);
        }
        set groupHeaderTextColor(v) {
            this.i.hh = stringToBrush(v);
            this._a("groupHeaderTextColor", brushToString(this.i.hh));
        }
        get actualGroupHeaderTextColor() {
            return brushToString(this.i.gw);
        }
        set actualGroupHeaderTextColor(v) {
            this.i.gw = stringToBrush(v);
            this._a("actualGroupHeaderTextColor", brushToString(this.i.gw));
        }
        /**
         * Gets or sets the text color for group header subtitle.
        */
        get groupHeaderSubtitleTextColor() {
            return brushToString(this.i.hg);
        }
        set groupHeaderSubtitleTextColor(v) {
            this.i.hg = stringToBrush(v);
            this._a("groupHeaderSubtitleTextColor", brushToString(this.i.hg));
        }
        get actualGroupHeaderSubtitleTextColor() {
            return brushToString(this.i.gv);
        }
        set actualGroupHeaderSubtitleTextColor(v) {
            this.i.gv = stringToBrush(v);
            this._a("actualGroupHeaderSubtitleTextColor", brushToString(this.i.gv));
        }
        /**
         * Gets or sets the tooltip text color.
        */
        get toolTipTextColor() {
            return brushToString(this.i.hs);
        }
        set toolTipTextColor(v) {
            this.i.hs = stringToBrush(v);
            this._a("toolTipTextColor", brushToString(this.i.hs));
        }
        get actualToolTipTextColor() {
            return brushToString(this.i.g7);
        }
        set actualToolTipTextColor(v) {
            this.i.g7 = stringToBrush(v);
            this._a("actualToolTipTextColor", brushToString(this.i.g7));
        }
        /**
         * Gets or sets the tooltip background color.
        */
        get toolTipBackgroundColor() {
            return brushToString(this.i.hr);
        }
        set toolTipBackgroundColor(v) {
            this.i.hr = stringToBrush(v);
            this._a("toolTipBackgroundColor", brushToString(this.i.hr));
        }
        /**
         * Gets the actual tooltip background color.
        */
        get actualToolTipBackgroundColor() {
            return brushToString(this.i.g6);
        }
        set actualToolTipBackgroundColor(v) {
            this.i.g6 = stringToBrush(v);
            this._a("actualToolTipBackgroundColor", brushToString(this.i.g6));
        }
        /**
         * Gets or sets the tooltip corner radius.
        */
        get toolTipCornerRadius() {
            return this.i.b2;
        }
        set toolTipCornerRadius(v) {
            this.i.b2 = +v;
            this._a("toolTipCornerRadius", this.i.b2);
        }
        /**
         * Gets the actual tooltip corner radius.
        */
        get actualToolTipCornerRadius() {
            return this.i.bq;
        }
        set actualToolTipCornerRadius(v) {
            this.i.bq = +v;
            this._a("actualToolTipCornerRadius", this.i.bq);
        }
        /**
         * Gets or sets the shadow elevation for the tooltip.
        */
        get toolTipElevation() {
            return this.i.b3;
        }
        set toolTipElevation(v) {
            this.i.b3 = +v;
            this._a("toolTipElevation", this.i.b3);
        }
        /**
         * Gets the actual shadow elevation used by the tooltip.
        */
        get actualToolTipElevation() {
            return this.i.br;
        }
        set actualToolTipElevation(v) {
            this.i.br = +v;
            this._a("actualToolTipElevation", this.i.br);
        }
        /**
         * Gets or sets the fill color of icons in the ToolPanel.
        */
        get iconFill() {
            return brushToString(this.i.hk);
        }
        set iconFill(v) {
            this.i.hk = stringToBrush(v);
            this._a("iconFill", brushToString(this.i.hk));
        }
        /**
         * Gets the actual icon fill color.
        */
        get actualIconFill() {
            return brushToString(this.i.gz);
        }
        set actualIconFill(v) {
            this.i.gz = stringToBrush(v);
            this._a("actualIconFill", brushToString(this.i.gz));
        }
        /**
         * Gets or sets the stroke color of icons in the ToolPanel.
        */
        get iconStroke() {
            return brushToString(this.i.hl);
        }
        set iconStroke(v) {
            this.i.hl = stringToBrush(v);
            this._a("iconStroke", brushToString(this.i.hl));
        }
        /**
         * Gets the actual icon stroke color.
        */
        get actualIconStroke() {
            return brushToString(this.i.g0);
        }
        set actualIconStroke(v) {
            this.i.g0 = stringToBrush(v);
            this._a("actualIconStroke", brushToString(this.i.g0));
        }
        /**
         * Gets or sets the left padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingLeft() {
            return this.i.bz;
        }
        set separatorVerticalPaddingLeft(v) {
            this.i.bz = +v;
            this._a("separatorVerticalPaddingLeft", this.i.bz);
        }
        get actualSeparatorVerticalPaddingLeft() {
            return this.i.bn;
        }
        set actualSeparatorVerticalPaddingLeft(v) {
            this.i.bn = +v;
            this._a("actualSeparatorVerticalPaddingLeft", this.i.bn);
        }
        /**
         * Gets or sets the right padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingRight() {
            return this.i.b0;
        }
        set separatorVerticalPaddingRight(v) {
            this.i.b0 = +v;
            this._a("separatorVerticalPaddingRight", this.i.b0);
        }
        get actualSeparatorVerticalPaddingRight() {
            return this.i.bo;
        }
        set actualSeparatorVerticalPaddingRight(v) {
            this.i.bo = +v;
            this._a("actualSeparatorVerticalPaddingRight", this.i.bo);
        }
        /**
         * Gets or sets the top padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingTop() {
            return this.i.b1;
        }
        set separatorVerticalPaddingTop(v) {
            this.i.b1 = +v;
            this._a("separatorVerticalPaddingTop", this.i.b1);
        }
        get actualSeparatorVerticalPaddingTop() {
            return this.i.bp;
        }
        set actualSeparatorVerticalPaddingTop(v) {
            this.i.bp = +v;
            this._a("actualSeparatorVerticalPaddingTop", this.i.bp);
        }
        /**
         * Gets or sets the bottom padding for separators in the vertical orientation.
        */
        get separatorVerticalPaddingBottom() {
            return this.i.by;
        }
        set separatorVerticalPaddingBottom(v) {
            this.i.by = +v;
            this._a("separatorVerticalPaddingBottom", this.i.by);
        }
        get actualSeparatorVerticalPaddingBottom() {
            return this.i.bm;
        }
        set actualSeparatorVerticalPaddingBottom(v) {
            this.i.bm = +v;
            this._a("actualSeparatorVerticalPaddingBottom", this.i.bm);
        }
        /**
         * Gets or sets the left padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingLeft() {
            return this.i.bv;
        }
        set separatorHorizontalPaddingLeft(v) {
            this.i.bv = +v;
            this._a("separatorHorizontalPaddingLeft", this.i.bv);
        }
        get actualSeparatorHorizontalPaddingLeft() {
            return this.i.bj;
        }
        set actualSeparatorHorizontalPaddingLeft(v) {
            this.i.bj = +v;
            this._a("actualSeparatorHorizontalPaddingLeft", this.i.bj);
        }
        /**
         * Gets or sets the right padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingRight() {
            return this.i.bw;
        }
        set separatorHorizontalPaddingRight(v) {
            this.i.bw = +v;
            this._a("separatorHorizontalPaddingRight", this.i.bw);
        }
        get actualSeparatorHorizontalPaddingRight() {
            return this.i.bk;
        }
        set actualSeparatorHorizontalPaddingRight(v) {
            this.i.bk = +v;
            this._a("actualSeparatorHorizontalPaddingRight", this.i.bk);
        }
        /**
         * Gets or sets the top padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingTop() {
            return this.i.bx;
        }
        set separatorHorizontalPaddingTop(v) {
            this.i.bx = +v;
            this._a("separatorHorizontalPaddingTop", this.i.bx);
        }
        get actualSeparatorHorizontalPaddingTop() {
            return this.i.bl;
        }
        set actualSeparatorHorizontalPaddingTop(v) {
            this.i.bl = +v;
            this._a("actualSeparatorHorizontalPaddingTop", this.i.bl);
        }
        /**
         * Gets or sets the bottom padding for separators in the horizontal orientation.
        */
        get separatorHorizontalPaddingBottom() {
            return this.i.bu;
        }
        set separatorHorizontalPaddingBottom(v) {
            this.i.bu = +v;
            this._a("separatorHorizontalPaddingBottom", this.i.bu);
        }
        get actualSeparatorHorizontalPaddingBottom() {
            return this.i.bi;
        }
        set actualSeparatorHorizontalPaddingBottom(v) {
            this.i.bi = +v;
            this._a("actualSeparatorHorizontalPaddingBottom", this.i.bi);
        }
        /**
         * Gets or sets the color to use for the background of the component.
        */
        get orientation() {
            return this.i.r;
        }
        set orientation(v) {
            this.i.r = ensureEnum(ToolPanelOrientation_$type, v);
            this._a("orientation", enumToString(ToolPanelOrientation_$type, this.i.r));
        }
        /**
         * Gets or sets the row height for the panel.
        */
        get rowHeight() {
            return this.i.b8;
        }
        set rowHeight(v) {
            this.i.b8 = +v;
            this._a("rowHeight", this.i.b8);
        }
        /**
         * Gets the actual row height used by the panel.
        */
        get actualRowHeight() {
            return this.i.bh;
        }
        get highlightWidth() {
            return this.i.bt;
        }
        set highlightWidth(v) {
            this.i.bt = +v;
            this._a("highlightWidth", this.i.bt);
        }
        get actualHighlightWidth() {
            return this.i.bg;
        }
        set actualHighlightWidth(v) {
            this.i.bg = +v;
            this._a("actualHighlightWidth", this.i.bg);
        }
        get highlightRadius() {
            return this.i.bs;
        }
        set highlightRadius(v) {
            this.i.bs = +v;
            this._a("highlightRadius", this.i.bs);
        }
        get actualHighlightRadius() {
            return this.i.bf;
        }
        set actualHighlightRadius(v) {
            this.i.bf = +v;
            this._a("actualHighlightRadius", this.i.bf);
        }
        get highlightColor() {
            return brushToString(this.i.hi);
        }
        set highlightColor(v) {
            this.i.hi = stringToBrush(v);
            this._a("highlightColor", brushToString(this.i.hi));
        }
        get actualHighlightColor() {
            return brushToString(this.i.gx);
        }
        set actualHighlightColor(v) {
            this.i.gx = stringToBrush(v);
            this._a("actualHighlightColor", brushToString(this.i.gx));
        }
        /**
         * Gets or sets the stroke color for all icon menu dropdown arrows.
        */
        get menuArrowStroke() {
            return brushToString(this.i.hm);
        }
        set menuArrowStroke(v) {
            this.i.hm = stringToBrush(v);
            this._a("menuArrowStroke", brushToString(this.i.hm));
        }
        /**
         * Gets the actual stroke color for all icon menu dropdown arrows.
        */
        get actualMenuArrowStroke() {
            return brushToString(this.i.g1);
        }
        set actualMenuArrowStroke(v) {
            this.i.g1 = stringToBrush(v);
            this._a("actualMenuArrowStroke", brushToString(this.i.g1));
        }
        /**
         * Gets or sets the background color of the radio when in the unchecked state.
        */
        get uncheckedBackgroundColor() {
            return brushToString(this.i.ht);
        }
        set uncheckedBackgroundColor(v) {
            this.i.ht = stringToBrush(v);
            this._a("uncheckedBackgroundColor", brushToString(this.i.ht));
        }
        /**
         * Gets the actual background color of the radio when in the unchecked state.
        */
        get actualUncheckedBackgroundColor() {
            return brushToString(this.i.g8);
        }
        set actualUncheckedBackgroundColor(v) {
            this.i.g8 = stringToBrush(v);
            this._a("actualUncheckedBackgroundColor", brushToString(this.i.g8));
        }
        /**
         * Gets or sets the background color of the radio when in the checked state.
        */
        get checkedBackgroundColor() {
            return brushToString(this.i.hb);
        }
        set checkedBackgroundColor(v) {
            this.i.hb = stringToBrush(v);
            this._a("checkedBackgroundColor", brushToString(this.i.hb));
        }
        /**
         * Gets the actual background color of the radio when in the checked state.
        */
        get actualCheckedBackgroundColor() {
            return brushToString(this.i.gq);
        }
        set actualCheckedBackgroundColor(v) {
            this.i.gq = stringToBrush(v);
            this._a("actualCheckedBackgroundColor", brushToString(this.i.gq));
        }
        /**
         * Gets or sets the border color of the radio when in the unchecked state.
        */
        get uncheckedBorderColor() {
            return brushToString(this.i.hu);
        }
        set uncheckedBorderColor(v) {
            this.i.hu = stringToBrush(v);
            this._a("uncheckedBorderColor", brushToString(this.i.hu));
        }
        /**
         * Gets the actual border color of the radio when in the unchecked state.
        */
        get actualUncheckedBorderColor() {
            return brushToString(this.i.g9);
        }
        set actualUncheckedBorderColor(v) {
            this.i.g9 = stringToBrush(v);
            this._a("actualUncheckedBorderColor", brushToString(this.i.g9));
        }
        /**
         * Gets or sets the border color of the radio when in the checked state.
        */
        get checkedBorderColor() {
            return brushToString(this.i.hc);
        }
        set checkedBorderColor(v) {
            this.i.hc = stringToBrush(v);
            this._a("checkedBorderColor", brushToString(this.i.hc));
        }
        /**
         * Gets the actual border color of the radio when in the checked state.
        */
        get actualCheckedBorderColor() {
            return brushToString(this.i.gr);
        }
        set actualCheckedBorderColor(v) {
            this.i.gr = stringToBrush(v);
            this._a("actualCheckedBorderColor", brushToString(this.i.gr));
        }
        /**
         * Gets or sets the dropdown delay time in milliseconds.
        */
        get dropdownDelay() {
            return this.i.b7;
        }
        set dropdownDelay(v) {
            this.i.b7 = +v;
            this._a("dropdownDelay", this.i.b7);
        }
        /**
         * Gets the actual dropdown delay.
        */
        get actualDropdownDelay() {
            return this.i.b6;
        }
        set actualDropdownDelay(v) {
            this.i.b6 = +v;
            this._a("actualDropdownDelay", this.i.b6);
        }
        /**
         * Gets or sets the mode used when displaying nested actions.
        */
        get nestedActionMode() {
            return this.i.e;
        }
        set nestedActionMode(v) {
            this.i.e = ensureEnum(NestedActionMode_$type, v);
            this._a("nestedActionMode", enumToString(NestedActionMode_$type, this.i.e));
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            if (this.actions != null && this.actions.findByName && this.actions.findByName(name)) {
                return this.actions.findByName(name);
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("ToolPanelComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        refresh() {
            this.i.fb();
        }
        getDesiredSize() {
            let iv = this.i.hv();
            return fromSize(iv);
        }
        /**
         * Forces any open sub menus to close.
        
        */
        closeSubmenus() {
            this.i.er();
        }
        exportVisualData() {
            let iv = this.i.b9();
            return (iv);
        }
        exportSerializedVisualData() {
            let iv = this.i.dl();
            return (iv);
        }
        get contentRefreshed() {
            return this._contentRefreshed;
        }
        set contentRefreshed(ev) {
            if (this._contentRefreshed_wrapped !== null) {
                this.i.contentRefreshed = delegateRemove(this.i.contentRefreshed, this._contentRefreshed_wrapped);
                this._contentRefreshed_wrapped = null;
                this._contentRefreshed = null;
            }
            this._contentRefreshed = ev;
            this._contentRefreshed_wrapped = (o, e) => {
                let outerArgs = new IgcToolPanelContentRefreshedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeContentRefreshed) {
                    this.beforeContentRefreshed(this, outerArgs);
                }
                if (this._contentRefreshed) {
                    this._contentRefreshed(this, outerArgs);
                }
            };
            this.i.contentRefreshed = delegateCombine(this.i.contentRefreshed, this._contentRefreshed_wrapped);
            ;
        }
        get onCommand() {
            return this._onCommand;
        }
        set onCommand(ev) {
            if (this._onCommand_wrapped !== null) {
                this.i.onCommand = delegateRemove(this.i.onCommand, this._onCommand_wrapped);
                this._onCommand_wrapped = null;
                this._onCommand = null;
            }
            this._onCommand = ev;
            this._onCommand_wrapped = (o, e) => {
                let outerArgs = new IgcToolCommandEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnCommand) {
                    this.beforeOnCommand(this, outerArgs);
                }
                if (this._onCommand) {
                    this._onCommand(this, outerArgs);
                }
            };
            this.i.onCommand = delegateCombine(this.i.onCommand, this._onCommand_wrapped);
            ;
        }
    }
    IgcToolPanelComponent._observedAttributesIgcToolPanelComponent = null;
    IgcToolPanelComponent.htmlTagName = "igc-tool-panel";
    IgcToolPanelComponent._isElementRegistered = false;
    return IgcToolPanelComponent;
})();
